@@ -1,6 +1,6 @@
 package Net::SFTP::Foreign;
 
-our $VERSION = '0.58';
+our $VERSION = '0.59';
 
 use strict;
 use warnings;
@@ -15,6 +15,13 @@ use Net::SFTP::Foreign::Buffer;
 
 use constant COPY_SIZE => 32768;
 
+our $dirty_cleanup;
+BEGIN {
+    if ($^O =~ /solaris/i) {
+	$dirty_cleanup = 1 unless defined $dirty_cleanup;
+    }
+}
+
 sub new {
     my $class = shift;
     unshift @_, 'host' if @_ & 1;
@@ -65,18 +72,34 @@ sub new {
     $sftp
 }
 
-
 sub DESTROY {
     my $sftp = shift;
-    if (defined $sftp->{pid}) {
+    my $pid = $sftp->{pid};
+    if (defined $pid) {
 	close $sftp->{ssh_out} if defined $sftp->{ssh_out};
 	close $sftp->{ssh_in} if defined $sftp->{ssh_in};
-	if ($^O =~ /Win/) {
-	    kill 1, $sftp->{pid}
-		and waitpid($sftp->{pid}, 0);
+	if ($dirty_cleanup) {
+	    for my $sig (1, 1, 1, 9, 9) {
+		kill $sig, $pid;
+		eval {
+		    local $SIG{ALRM} = sub { die "timeout\n" };
+		    alarm 2;
+		    waitpid($pid, 0);
+		    alarm 0;
+		};
+		if ($@) {
+		    next if $@ =~ /^timeout/;
+		    die $@;
+		}
+		last;
+	    }
+	}
+	elsif ($^O =~ /mswin/i) {
+	    kill 1, $pid
+		and waitpid($pid, 0);
 	}
 	else {
-	    waitpid($sftp->{pid}, 0);
+	    waitpid($pid, 0);
 	}
     }
 }
@@ -928,7 +951,15 @@ function in Net::SFTP::Foreign::Util.
 
 =head1 BUGS
 
-This module has been minimally tested. Use with care!
+On some operative systems, closing the pipes used to comunicate with
+the slave ssh process does not terminate it and a work around has to
+be applied. If you find that your scripts hung when the sftp object
+gets out of scope, try setting C<$Net::SFTP::Foreign::dirty_cleanup>
+to a true value and also send me a report including the value of
+C<$^O> on your machine.
+
+To report bugs, please, send me and email or use
+L<http://rt.cpan.org>.
 
 
 =head1 SEE ALSO
